//
// Executioner.cpp
//
// Copyright Howard Berkey, 1997
//
//    Please see the README file for usage info. 
//
// The usual doom and gloom warnings apply, there is no implied warrantee 
// on this code, I'm not responsible for any lost data caused by using this add-on, 
// I do not guarantee it will not accidently murder your pets, etc.
//
// To install, put the shared library ("Executioner") in the BeIDE 
// /boot/apps/Metrowerks/plugins/Editor_add_ons folder.
//
// To build this, simply open the project and do a make.
// If you'd like to learn how to do a BeIDE add on from scratch, follow these steps:
//
// Make a new project.  Add the .cpp files to it.
//
// In the project settings, set:
//
// Project::PPC Project::Project Type = Shared Library
// Project::PPC Project::File Name = Executioner (or whatever you like, really)
// Project::Access Paths add the path /boot/apps/Metrowerks/examples/Plugin_API/MWEditorAddOns
// Linker::Linker::Main  blank this field out
// Linker::PEF::Export Symbols = Use #pragma
//
// Optional:
// Language::C/C++ Warnings = all on, apply to new projects, save (you'll thank me later :-)
//
// Then do a Make, and put the shared library ("Executioner") in the BeIDE 
// "/boot/.../Metrowerks/plugins/Editor_add_ons" folder (... is usually "apps").  
// Restart BeIDE.
//
//
// Main program by:
// Howard Berkey
// hberkey@ten.net     
//
// Unix tool output processing and other features and extensions by:
// Timothy James
// tjames@nmsu.edu 
//


#include "MTextAddOn.h"
#include <stdio.h>
#include <string.h>

#include "Executioner.h"

#pragma export on
extern "C" status_t perform_edit(MTextAddOn *mtex);
#pragma export off

#define BUF_SIZ 1024

// exported add-on interface
extern "C" status_t perform_edit(MTextAddOn *mtex)
{
	long s_start, s_end;
	const char *txtbuf;
	char *cmdbuf, retbuf[BUF_SIZ];
	FILE *pf;
	bool  rc, wasAtEOF=false;
	
	if(!mtex)
	{
		beep();
		return B_ERROR;
	}
	
	//
	// Get the range of selected text
	//
	mtex->GetSelection(&s_start, &s_end);
	
	//
	// Make sure there's no error in the selection
	//
	if(s_end < 0 || s_start < 0 || s_end < s_start)
	{
		beep();
		return B_ERROR;
	}
	
	//
	// Get a pointer to the text 
	//
	txtbuf = mtex->Text();
	
	//
	// If nothing is selected, we select the line the cursor is on
	//
	if(s_start == s_end)
	{
		while(s_start > 0 && txtbuf[s_start - 1] != '\n')
			s_start--;
		
		while(s_end < mtex->TextLength() && txtbuf[s_end] != '\n')
			s_end++;

		if (s_end != mtex->TextLength())
			s_end++; // include the '\n'
		else
			wasAtEOF = true;
				
		mtex->Select(s_start, s_end);			
	}
			
	//
	// Copy the selected text into our cmdbuf
	//

	txtbuf += s_start;	
	cmdbuf = new char[(s_end - s_start) + 10];  // enough for stderr redirect
	strncpy(cmdbuf, txtbuf, s_end - s_start);
	cmdbuf[s_end - s_start] = '\0';
	
	if (cmdbuf[s_end - s_start - 1] == '\n')
		cmdbuf[s_end - s_start - 1] = '\0';

	// 
	// Here's the whizzy part of all this.  It would be really cool to 
	// be able to detect some non-shell script syntax and parse it correctly.
	// We put that hook here.  It would be cool to make this dynamically
	// loadable code from within this plugin using shared libraries to
	// provide "Executioner Parser Plugins" but I don't have that kind of time
	// to throw at this :-)
	//
	// If parseHook returns false, we do not continue.
	//
	
	rc = parseHook(mtex, &cmdbuf);
	
	if(rc == false)
	{
		delete [] cmdbuf;
		return B_NO_ERROR;
	}
			
	//
	// Execute the command in the shell, and free the command buffer.
	// First redirect stderr on the LAST COMMAND ONLY, unless control
	// key is down.	
	//
	if (!(modifiers() & B_CONTROL_KEY))
		strcat(cmdbuf, " 2>&1");

	pf = popen(cmdbuf, "r");	
	delete [] cmdbuf;
	
	// 
	// Unless there was an error in popen, pf should point to the output 
	// of the shell command
	//
	if(!pf)
	{
		beep();
		return B_ERROR;
	}
	
	//
	// Move the insertion point to just after the text that 
	// we just executed.  This will be the start of the output
	// which we will leave selected.
	// Insert a new line if we are at EOF.
	//	
	s_start = s_end;
	if (wasAtEOF)
	{
		s_start = s_end++;
		mtex->Select(s_start, s_start);		
		mtex->Insert("\n");   // start new line!
		s_start = s_end++;
	}	
	mtex->Select(s_start, s_start);		

		

	//
	// Read the output of the shell command until we get it all.
	// Insert each chunk into the BeIDE text window as we get it.
	//
	key_info keyInfo;	
	while(fgets(retbuf, sizeof(retbuf), pf))
	{
		mtex->Insert(retbuf);
		s_end += strlen(retbuf);
		
		// is the escape key down?
		if  ( (B_OK == get_key_info(&keyInfo)) &&    
		      (keyInfo.key_states[0] & 0x40  )     ) 
		{
			mtex->Insert(         "    //// CANCELLED ////\n");
			s_end += 24; //strlen("    //// CANCELLED ////\n");
			break;
		}
	}
	
	//
	// Clean up and select the text we just inserted.  
	// We're done!
	//

	mtex->Select(s_start, s_end);

	fclose(pf);	
	return B_NO_ERROR;		
}


